<?php

namespace Drupal\drush_block\Drush\Commands;

use Consolidation\OutputFormatters\StructuredData\RowsOfFields;
use Drupal\block\BlockInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drush\Attributes as CLI;
use Psr\Container\ContainerInterface as DrushContainer;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Utility to list, enable and disable blocks using Drush.
 */
class DrushBlockDrushCommands {

  use StringTranslationTrait;

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $etm
   *   The entity-type manager.
   */
  public function __construct(protected EntityTypeManagerInterface $etm) {
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, DrushContainer $drush): self {
    return new static(
      $container->get('entity_type.manager'),
    );
  }

  /**
   * List the blocks.
   */
  #[CLI\Command(name: 'block:list')]
  #[CLI\FieldLabels(labels: [
    'id' => 'ID',
    'label' => 'Label',
    'theme' => 'Theme',
    'region' => 'Region',
    'status' => 'Status',
  ])]
  public function list($options = ['format' => 'table']) {
    $blocks = $this
      ->etm
      ->getStorage('block')
      ->loadMultiple();

    $blocks = array_map([$this, 'getDefinition'], $blocks);
    return new RowsOfFields($blocks);
  }

  /**
   * Disable a block.
   *
   * @param string $id
   *   The ID of the block entity.
   */
  #[CLI\Command(name: 'block:disable')]
  public function disable(string $id) : void {
    /** @var \Drupal\block\BlockInterface */
    $block = $this
      ->etm
      ->getStorage('block')
      ->load($id);

    if (!$block) {
      throw new \RuntimeException('Block not found');
    }

    $block
      ->disable()
      ->save();
  }

  /**
   * Enable a block.
   *
   * @param string $id
   *   The ID of the block entity.
   */
  #[CLI\Command(name: 'block:enable')]
  public function enable(string $id) : void {
    /** @var \Drupal\block\BlockInterface */
    $block = $this
      ->etm
      ->getStorage('block')
      ->load($id);

    if (!$block) {
      throw new \RuntimeException('Block not found');
    }
    $block
      ->enable()
      ->save();
  }

  /**
   * Get the fields exposed to Drush.
   *
   * @param \Drupal\block\BlockInterface $block
   *   The block entity.
   *
   * @return array
   *   The fields to provide to Drush,
   */
  protected function getDefinition(BlockInterface $block) {
    return [
      'id' => $block->id(),
      'label' => $block->label(),
      'theme' => $block->getTheme(),
      'region' => $block->getRegion(),
      'status' => $block->status() ? $this->t('Enabled') : $this->t('Disabled'),
    ];
  }

}
